-------------------------------------------------------------------------------
--! @file       COLM.vhd
--! @author     Ahmed Ferozpuri
--! @brief      MixColumns for PRIMATEs
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY COLM IS
	GENERIC (C : INTEGER := 25;
				isINV : INTEGER := 0);
	PORT(	C_i : IN  std_logic_vector(C-1 DOWNTO 0);
			C_o : OUT  std_logic_vector(4 DOWNTO 0)
		);
END COLM;

ARCHITECTURE behavioral OF COLM IS

	TYPE tType IS ARRAY (0 TO C/5) OF std_logic_vector (4 DOWNTO 0); 
	SIGNAL T : tType;
BEGIN
		
	COLMP80:IF C=25 GENERATE
		GENT:IF isINV=0 GENERATE
			T(0) <= C_i(24 DOWNTO 20);
			T2O: ENTITY work.GF5M18(behavioral) PORT MAP(d_i => C_i(19 DOWNTO 15), d_o => T(1));
			T3O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(14 DOWNTO 10), d_o => T(2));
			T4O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(9 DOWNTO 5), d_o => T(3));
			T5O: ENTITY work.GF5M18(behavioral) PORT MAP(d_i => C_i(4 DOWNTO 0), d_o => T(4));
		END GENERATE GENT;
		GENTINV:IF isINV=1 GENERATE
			T1O: ENTITY work.GF5M18(behavioral) PORT MAP(d_i => C_i(24 DOWNTO 20), d_o => T(0));
			T2O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(19 DOWNTO 15), d_o => T(1));
			T3O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(14 DOWNTO 10), d_o => T(2));
			T4O: ENTITY work.GF5M18(behavioral) PORT MAP(d_i => C_i(9 DOWNTO 5), d_o => T(3));
			T(4) <= C_i(4 DOWNTO 0);
		END GENERATE GENTINV;
		GEN_d_o: FOR i IN 0 TO 4 GENERATE
			C_o(i) <= T(0)(i) xor T(1)(i) xor T(2)(i) xor T(3)(i) xor T(4)(i);
		END GENERATE GEN_d_o;
	END GENERATE;
	
	COLMP120:IF C=35 GENERATE
		GENT:IF isINV=0 GENERATE
			T(0) <= C_i(34 DOWNTO 30);
			T2O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(29 DOWNTO 25), d_o => T(1));
			T3O: ENTITY work.GF5M15(behavioral) PORT MAP(d_i => C_i(24 DOWNTO 20), d_o => T(2));
			T4O: ENTITY work.GF5M9(behavioral) PORT MAP(d_i => C_i(19 DOWNTO 15), d_o => T(3));
			T5O: ENTITY work.GF5M9(behavioral) PORT MAP(d_i => C_i(14 DOWNTO 10), d_o => T(4));
			T6O: ENTITY work.GF5M15(behavioral) PORT MAP(d_i => C_i(9 DOWNTO 5), d_o => T(5));
			T7O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(4 DOWNTO 0), d_o => T(6));
		END GENERATE GENT;
		GENTINV:IF isINV=1 GENERATE
			T1O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(34 DOWNTO 30), d_o => T(0));
			T2O: ENTITY work.GF5M15(behavioral) PORT MAP(d_i => C_i(29 DOWNTO 25), d_o => T(1));
			T3O: ENTITY work.GF5M9(behavioral) PORT MAP(d_i => C_i(24 DOWNTO 20), d_o => T(2));
			T4O: ENTITY work.GF5M9(behavioral) PORT MAP(d_i => C_i(19 DOWNTO 15), d_o => T(3));
			T5O: ENTITY work.GF5M15(behavioral) PORT MAP(d_i => C_i(14 DOWNTO 10), d_o => T(4));
			T6O: ENTITY work.GF5M2(behavioral) PORT MAP(d_i => C_i(9 DOWNTO 5), d_o => T(5));
			T(6) <= C_i(4 DOWNTO 0);
		END GENERATE GENTINV;
		GEN_d_o: FOR i IN 0 TO 4 GENERATE
			C_o(i) <= T(0)(i) xor T(1)(i) xor T(2)(i) xor T(3)(i) xor T(4)(i) xor T(5)(i) xor T(6)(i);
		END GENERATE GEN_d_o;
	END GENERATE;
END behavioral;