-------------------------------------------------------------------------------
--! @file       SR.vhd
--! @author     Ahmed Ferozpuri
--! @brief      ShiftRows (SR) for PRIMATEs
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY SR IS
	GENERIC (V : INTEGER := 200;
				isINV : INTEGER := 0);
	PORT(	V_i : IN  std_logic_vector(V-1 DOWNTO 0);
			V_o : OUT  std_logic_vector(V-1 DOWNTO 0)
		);
END SR;

ARCHITECTURE behavioral OF SR IS
BEGIN

	V_o(V-1 DOWNTO V-40) <= V_i(V-1 DOWNTO V-40); --rotate row 0 by 0

	GENSR:IF isINV=0 GENERATE	
		V_o(V-41 DOWNTO V-80) <= V_i(V-46 DOWNTO V-80) & V_i(V-41 DOWNTO V-45); --rotate left row 1 by 1 cell
		V_o(V-81 DOWNTO V-120) <= V_i(V-91 DOWNTO V-120) & V_i(V-81 DOWNTO V-90); --rotate left row 2 by 2 cells
		V_o(39 DOWNTO 0) <= V_i(4 DOWNTO 0) & V_i(39 DOWNTO 5); --rotate left row 6 by 7	or rotate left row 4 by 7 cells		
	
		SRP80:IF V=200 GENERATE
			V_o(79 DOWNTO 40) <= V_i(59 DOWNTO 40) & V_i(79 DOWNTO 60); --rotate left row 3 by 4 cells
		END GENERATE;
		
		SRP120:IF V=280 GENERATE
			V_o(159 DOWNTO 120) <= V_i(144 DOWNTO 120) & V_i(159 DOWNTO 145); --rotate left row 3 by 3
			V_o(119 DOWNTO 80) <= V_i(99 DOWNTO 80) & V_i(119 DOWNTO 100); --rotate left row 4 by 4
			V_o(79 DOWNTO 40) <= V_i(54 DOWNTO 40) & V_i(79 DOWNTO 55); --rotate left row 5 by 5
		END GENERATE;
	END GENERATE GENSR;
	
	GENINVSR:IF isINV=1 GENERATE
		V_o(V-41 DOWNTO V-80) <= V_i(V-76 DOWNTO V-80) & V_i(V-41 DOWNTO V-75); --rotate right row 1 by 1 cell
		V_o(V-81 DOWNTO V-120) <= V_i(V-111 DOWNTO V-120) & V_i(V-81 DOWNTO V-110); --rotate right row 2 by 2 cells
		V_o(39 DOWNTO 0) <= V_i(34 DOWNTO 0) & V_i(39 DOWNTO 35); --rotate right row 6 by 7 or rotate right row 4 by 7 cells
		
		SRP80:IF V=200 GENERATE
			V_o(79 DOWNTO 40) <= V_i(59 DOWNTO 40) & V_i(79 DOWNTO 60); --rotate right row 3 by 4 cells
		END GENERATE;
		
		SRP120:IF V=280 GENERATE
			V_o(159 DOWNTO 120) <= V_i(134 DOWNTO 120) & V_i(159 DOWNTO 135); --rotate right row 3 by 3
			V_o(119 DOWNTO 80) <= V_i(99 DOWNTO 80) & V_i(119 DOWNTO 100); --rotate right row 4 by 4
			V_o(79 DOWNTO 40) <= V_i(64 DOWNTO 40) & V_i(79 DOWNTO 65); --rotate right row 5 by 5
		END GENERATE;
	END GENERATE GENINVSR;
END behavioral;